#readelf: --sym
#name: ia64 global label

Symbol table '.symtab' contains 5 entries:
   Num:    Value          Size Type    Bind   Vis      Ndx Name
     0: 0+0     0 NOTYPE  LOCAL  DEFAULT  UND 
     1: 0+0     0 SECTION LOCAL  DEFAULT    1 
     2: 0+0     0 SECTION LOCAL  DEFAULT    2 
     3: 0+0     0 SECTION LOCAL  DEFAULT    3 
     4: 0+0     0 NOTYPE  GLOBAL DEFAULT    2 foo
