#as: --isa=shmedia --abi=64 --no-exp -little
#objdump: -s
#name: SH64 Little Endian

.*:     file format elf64-sh64.*

Contents of section .text:
 0000 00d048cc 78563412 34120000.*
Contents of section .data:
